import 'dart:convert';
import 'dart:io';

class ScHttpClient {
  var _client = HttpClient();
  String Function(String) getCache;
  void Function(String, String, Duration) setCache;

  ScHttpClient(this.getCache, this.setCache);

  Future<String> post(
    Uri url,
    Object body,
    String id,
    Map<String, String> headers, {
    Duration ttl,
  }) async {
    ttl ??= Duration(minutes: 15);
    if (getCache != null) {
      var cachedResp = getCache(id);
      if (cachedResp != null) return cachedResp;
    }
    var req = await _client.postUrl(url);
    headers.forEach((k, v) => req.headers.add(k, v));
    req.writeln(body);
    return _finishRequest(req, id, ttl);
  }

  Future<String> get(Uri url, {Duration ttl}) async {
    ttl ??= Duration(days: 4);
    if (getCache != null) {
      var cachedResp = getCache('$url');
      if (cachedResp != null) return cachedResp;
    }
    var req = await _client.getUrl(url);
    return _finishRequest(req, '$url', ttl);
  }

  Future<String> _finishRequest(
    HttpClientRequest req,
    String id,
    Duration ttl,
  ) async {
    await req.flush();
    var res = await req.close();
    var bytes = await res.toList();
    var actualBytes = <int>[];
    for (var b in bytes) actualBytes.addAll(b);
    var r = utf8.decode(actualBytes);
    if (res.statusCode == 200 && setCache != null) setCache(id, r, ttl);
    return r;
  }
}
